﻿using System.Text;

namespace Turing
{
    public class Stan
    {
        private char stanGłowicy;
        private int położenieGłowicy;
        //private string taśma;
        private StringBuilder taśma;

        public char StanGłowicy
        {
            get
            {
                return stanGłowicy;
            }
        }

        public char StanKomórkiPodGłowicą
        {
            get
            {
                return taśma[położenieGłowicy];
            }
        }

        public Stan(char stanGłowicy, int położenieGłowicy, string taśma)
        {
            this.stanGłowicy = stanGłowicy;
            this.położenieGłowicy = położenieGłowicy;
            //this.taśma = taśma;
            this.taśma = new StringBuilder(taśma);

            if (!char.IsLower(stanGłowicy)) throw new TuringException("Zły opis stanu głowicy");
            if (położenieGłowicy < 0 || położenieGłowicy >= taśma.Length) throw new TuringException("Niepoprawne położenie głowicy");
            //TODO: skopiowany kod!!!
            if (!taśma.All(c => char.IsUpper(c))) throw new TuringException("Niepoprawny format taśmy");
        }

        public static Stan Wczytaj(string ścieżkaPliku)
        {
            string tekst = File.ReadAllText(ścieżkaPliku);
            //if (tekst.Count((char c) => { return char.IsLower(c); }) != 1) throw new TuringException("Błędny format pliku stanu maszyny - nieodpowiednia liczba znaków oznaczających stan głowicy");
            if (tekst.Count(c => char.IsLower(c)) != 1) throw new TuringException("Błędny format pliku stanu maszyny - nieodpowiednia liczba znaków oznaczających stan głowicy");            
            char stanGłowicy = tekst.First(c => char.IsLower(c));
            int położenieGłowicy = tekst.IndexOf(stanGłowicy);
            string taśma = tekst.Substring(0, położenieGłowicy) + tekst.Substring(położenieGłowicy + 1);
            if (!taśma.All(c => char.IsUpper(c))) throw new TuringException("Niepoprawny format taśmy");
            return new Stan(stanGłowicy, położenieGłowicy, taśma);
        }

        private void zmieńStanGłowicy(char nowyStanGłowicy)
        {
            if (!char.IsLower(nowyStanGłowicy)) throw new TuringException("Zły opis stanu głowicy");
            stanGłowicy = nowyStanGłowicy;
        }

        public void PrzesuńGłowicęWLewo(char nowyStanGłowicy)
        {
            if (położenieGłowicy - 1 < 0) throw new TuringException("Wyjście poza taśmę z lewej strony");
            else położenieGłowicy--;
            zmieńStanGłowicy(nowyStanGłowicy);
        }

        public void PrzesuńGłowicęWPrawo(char nowyStanGłowicy)
        {
            if (położenieGłowicy + 1 > taśma.Length) throw new TuringException("Wyjście poza taśmę z prawej strony");
            else położenieGłowicy++;
            zmieńStanGłowicy(nowyStanGłowicy);
        }

        public void ZmieńStanKomórkiPodGłowicą(char nowaWartośćKomórki, char nowyStanGłowicy)
        {
            if (!char.IsUpper(nowaWartośćKomórki)) throw new TuringException("Zły opis wartości komórki");
            //taśma = taśma.Substring(0, położenieGłowicy - 1) + nowaWartośćKomórki.ToString() + taśma.Substring(położenieGłowicy + 1);
            //taśma = taśma.Remove(położenieGłowicy, 1).Insert(położenieGłowicy, nowaWartośćKomórki.ToString());
            taśma.Remove(położenieGłowicy, 1).Insert(położenieGłowicy, nowaWartośćKomórki);
            zmieńStanGłowicy(nowyStanGłowicy);
        }

        public override string ToString()
        {            
            //return taśma.Insert(położenieGłowicy, stanGłowicy.ToString());
            return taśma.ToString().Insert(położenieGłowicy, stanGłowicy.ToString());
        }
    }
}
